'// Original WinDev/GDImage Scaffolding project (alias GLpixel),
'// translated to plain SDK flat API
'// Created by Patrice TERRIER on april 10, 2012
'// Last revision december 28, 2014

#COMPILE EXE "Scaffolding.exe"
#INCLUDE     "Windows.inc"
#INCLUDE     "GL.inc"
#INCLUDE     "GLU.inc"
#INCLUDE     "GDImage.inc"
#RESOURCE    "zapdpi.pbr"

#INCLUDE     "WinLIFT.inc"

declare function PathCombine lib "SHLWAPI.DLL" alias "PathCombineA" (byref lpszDest as asciiz, byref lpszDir as asciiz, byref lpszFile as asciiz) as long
declare function zTrace lib "zTrace.DLL" alias "zTrace" (zMessage as asciiz) as long

%TBS_AUTOTICKS = &H0001
%TBS_VERT      = &H0002
%TBS_BOTTOM    = &H0000
%TBM_GETPOS    = %WM_USER
%TBM_SETPOS    = %WM_USER + 5
%TBM_SETRANGE  = %WM_USER + 6
%WM_APPLYFOCUS = %WM_USER + 999

%SceneWidth    = 1330	'// Do not alter this value it is the maximum OpenGL scene width.
%SceneHeight   = 1330	'// Do not alter this value it is the maximum OpenGL scene height.

%ClientW       = 840
%ClientH       = 600

%IDC_OGLCTRL   = -1
%IDC_TIMER     = 1
%IDC_USINGIT   = 2
%IDC_LABEL     = 3
%IDC_ANIMATE   = 4
%IDC_ZOOMING   = 5
%IDC_RESTORE   = 6

type PROP
    usefont           as dword
    hinstance         as dword
    hMain             as dword
    hGL               as dword
    GLproc            as dword
    MinTrackSizeW     as long
    MinTrackSizeH     as long
    glfont            as ZGLFONT

    texBuilding       as long
    texChrome         as long
    texRadial         as long
    texRedwood        as long
    texFloor          as long
    texConcrete       as long
    texSpecular       as long

    MouseLastPos      as POINT
    MouseDown         as POINT
    LeftMousing       as long
    RightMousing      as long
    rXcopy            as single
    rYcopy            as single
    nXdown            as long
    nYdown            as long
    nWasXL            as long
    nWasYL            as long
    nWasXR            as long
    nWasYR            as long

    ColorWhite        as long
    ColorGreen        as long
    ColorYellow       as long
    ColorBlack        as long
    ColorRed          as long
    ColorDarkGray     as long
    ColorLightGray    as long
    ColorPastelYellow as long
end type

global gP as PROP, g_mt() as ZGLTEXTUREX

function dpi(byval nPix as long) as long
    static ratio as single
    if (ratio = 0) then
        local hDC as DWORD
        hDC = GetDC(%NULL): ratio = (GetDeviceCaps(hDC, %LOGPIXELSX) / 96): ReleaseDC(%NULL, hDC)
    end if
    function = clng(nPix * ratio)
end function

function addLabel(byval hParent as dword, byval nID as long, zCaption as asciiz, byval x as long, byval y as long, byval w as long, byval h as long, byval dwStyle as dword) as dword
    local hCtrl as dword
    hCtrl = CreateWindowEx(0, "Static", zCaption, dwStyle, dpi(x), dpi(y), dpi(w), dpi(h), hParent, nID, gP.hinstance, byval %NULL)
    if (hCtrl) then SendMessage(hCtrl, %WM_SETFONT, gP.usefont, 0)
    function = hCtrl
end function

function addButton(byval hParent as dword, byval nID as long, zCaption as asciiz, byval x as long, byval y as long, byval w as long, byval h as long, byval dwStyle as dword) as dword
    local hCtrl as dword
    hCtrl = CreateWindowEx(0, "Button", zCaption, dwStyle, dpi(x), dpi(y), dpi(w), dpi(h), hParent, nID, gP.hinstance, byval %NULL)
    if (hCtrl) then SendMessage(hCtrl, %WM_SETFONT, gP.usefont, 0)
    function = hCtrl
end function

function EXEpath() as string
    static zPath as asciiz * %MAX_PATH
    local nLen as long: nLen = len(zPath)
    if (nLen = 0) then
        if (GetModuleFileName(byval %NULL, zPath, sizeof(zPath))) then
            local pBits as byte ptr
            local K as long
            nLen = len(zPath)
            pBits = varptr(zPath) + nLen
            for K = 1 to nLen
                if (@pBits[-K] <> 92) then
                    @pBits[-K] = 0
                else
                    exit for
                end if
            next
        end if
    end if
    function = zPath
end function

function EXEresource() as string
    static szResource as asciiz * %MAX_PATH
    if (len(szResource) = 0) then PathCombine(szResource, EXEpath(), "resource\")
    function = szResource
end function

sub gl_DestroyTexture()
    local K, mtCount, OkDelete as long
    mtCount = UBOUND(g_mt()) - LBOUND(g_mt()) + 1
    if (mtCount) then
        dim Texture(mtCount - 1) as long
        for K = 0 TO mtCount - 1
            Texture(K) = g_mt(k).Texture: if (Texture(K)) then OkDelete = -1
        next
        if (OkDelete) then glDeleteTextures(mtCount, Texture(0))
    end if
end sub

'// Draw a circle.
'// Dessine un cercle.
sub gl_dCircle(byval rCenterX as single, byval rCenterY as single, byval rRadius as single, byval nARGB as long, byval rPenSize as single)
    local nAlpha, nRed, nGreen, nBlue as byte
    zSplitColorARGB(nARGB, nAlpha, nRed, nGreen, nBlue)
    glColor4ub(nRed, nGreen, nBlue, nAlpha)
    glLineWidth(rPenSize)

    glBegin(%GL_LINE_LOOP)
    local nAngle as long
    for nAngle = 0 to 360 step 5
        glVertex2f(rCenterX + (rRadius + sin(DegreesToRadians(nAngle)) * rRadius), _
                   rCenterY + (rRadius + cos(DegreesToRadians(nAngle)) * rRadius))
    next
    glEnd()
end sub

'// Draw a cup.
'// Dessine une coupe.
sub gl_dCup(byval rX as single, byval rY as single, byval rZ as single, byval rRadius as single, byval nARGB as long, byval nSphereTexture as long)
    local nAlpha, nRed, nGreen, nBlue as byte
    zSplitColorARGB(nARGB, nAlpha, nRed, nGreen, nBlue)
    glColor4ub(nRed, nGreen, nBlue, nAlpha)
    if (nSphereTexture) then
        glEnable(%GL_TEXTURE_2D)
        glBindTexture(%GL_TEXTURE_2D, nSphereTexture)
    end if
    glTranslatef(rX, rY, rZ)
    GL_DrawCup(rRadius, nSphereTexture)
    if (nSphereTexture) then glDisable(%GL_TEXTURE_2D)
end sub

'// Draw a dome.
'// Dessine un dome.
sub gl_dDome(byval rX as single, byval rY as single, byval rZ as single, byval rRadius as single, byval nARGB as long, byval nSphereTexture as long)
    local nAlpha, nRed, nGreen, nBlue as byte
    zSplitColorARGB(nARGB, nAlpha, nRed, nGreen, nBlue)
    glColor4ub(nRed, nGreen, nBlue, nAlpha)
    if (nSphereTexture) then
        glEnable(%GL_TEXTURE_2D)
        glBindTexture(%GL_TEXTURE_2D, nSphereTexture)
    end if
    glTranslatef(rX, rY, rZ)
    GL_DrawDome(rRadius, nSphereTexture)
    if (nSphereTexture) then glDisable(%GL_TEXTURE_2D)
end sub

'// Draw a sphere.
'// Dessine une sphere.
sub gl_dSphere(byval rX as single, byval rY as single, byval rZ as single, byval rRadius as single, byval nARGB as long, byval nSphereTexture as long)
    local nAlpha, nRed, nGreen, nBlue as byte
    zSplitColorARGB(nARGB, nAlpha, nRed, nGreen, nBlue)
    glColor4ub(nRed, nGreen, nBlue, nAlpha)
    if (nSphereTexture) then
        glEnable(%GL_TEXTURE_2D)
        glBindTexture(%GL_TEXTURE_2D, nSphereTexture)
    end if
    glTranslatef(rX, rY, rZ)
    GL_DrawSphere(rRadius, nSphereTexture)
    if (nSphereTexture) then glDisable(%GL_TEXTURE_2D)
end sub

'// Draw a torus.
'// Dessine un tore.
sub gl_dTorus(byval TORUS_MINOR as double, byval TORUS_MAJOR as double, byval TORUS_MINOR_RES as long, byval TORUS_MAJOR_RES as long, byval nARGB as long, byval nTexture as long)
    local nAlpha, nRed, nGreen, nBlue as byte
    zSplitColorARGB(nARGB, nAlpha, nRed, nGreen, nBlue)
    glColor4ub(nRed, nGreen, nBlue, nAlpha)
    if (nTexture) then
        glEnable(%GL_TEXTURE_2D)
        glBindTexture(%GL_TEXTURE_2D, nTexture)
    end if

    static nTorus_List as long
    if (nTorus_List = 0) then
        local nI, nJ, nK as long
        local rS, rT, rX, rY, rZ, rXx, rYy, rZz, rScale, rTwopi as double

        '// Start recording displaylist
        nTorus_List = glGenLists(1)
        glNewList(nTorus_List, %GL_COMPILE_AND_EXECUTE )

        '// Draw torus
        rTwopi = 2.0 * Pi
        for nI = 0 to TORUS_MINOR_RES - 1
            glBegin(%GL_QUAD_STRIP )
            for nJ = 0 to TORUS_MAJOR_RES
                for nK = 1 to 0 step -1

                    rS = ((nI + nK) mod TORUS_MINOR_RES) + 0.5
                    rT = (nJ mod TORUS_MAJOR_RES)

                    '// Calculate point on surface
                    rX = (TORUS_MAJOR + TORUS_MINOR * cos(rS * rTwopi / TORUS_MINOR_RES)) * cos(rT * rTwopi / TORUS_MAJOR_RES)
                    rY = TORUS_MINOR * sin(rS * rTwopi / TORUS_MINOR_RES)
                    rZ = (TORUS_MAJOR + TORUS_MINOR * cos(rS * rTwopi / TORUS_MINOR_RES)) * sin(rT * rTwopi / TORUS_MAJOR_RES)

                    '// Calculate surface normal
                    rXx = rX - TORUS_MAJOR * cos(rT * rTwopi / TORUS_MAJOR_RES)
                    rYy = rY
                    rZz = rZ - TORUS_MAJOR * sin(rT * rTwopi / TORUS_MAJOR_RES)
                    rScale = 1.0 / sqr (rXx * rXx + rYy * rYy + rZz * rZz )
                    rXx = rXx * rScale
                    rYy = rYy * rScale
                    rZz = rZz * rScale

                    glTexCoord2f(nJ / sqr(sqr(TORUS_MAJOR)), nI / sqr(sqr(TORUS_MINOR)))
                    glNormal3f(rXx, rYy, rZz)
                    glVertex3f(rX, rY, rZ)

                next
            next
            glEnd()
        next

        '// Stop recording displaylist
        glEndList()
    else
        '// Playback displaylist
        glCallList(nTorus_List)
    end if

    if (nTexture) then glDisable(%GL_TEXTURE_2D)
end sub

'// Draw an OpenGL line between two points using a specific ARGB color and pen size.
'// Trace une ligne OpenGL entre deux points en utilisant une couleur ARGB et un trait dont l'paisseur correspond  la taille du stylo .
sub gl_dLine(byval rX1 as single, byval rY1 as single, byval rX2 as single, byval rY2 as single, byval nARGB as long, byval rSize as single)
    local nAlpha, nRed, nGreen, nBlue as byte
    zSplitColorARGB(nARGB, nAlpha, nRed, nGreen, nBlue)
    glColor4ub(nRed, nGreen, nBlue, nAlpha)
    glLineWidth(rSize)
    glBegin(%GL_LINES)
      glVertex2f(rX1, rY1): glVertex2f(rX2, rY2)
    glEnd()
end sub

'// Draw an OpenGL outlined rectangle, which size matches the point coordinates, using a specific ARGB color and pen size.
'// Trace le contour d'un rectangle OpenGL dont la dimension est dtermine par les coordonnes des points opposs, en utilisant une couleur ARGB et un trait dont l'paisseur correspond  la taille du stylo.
sub gl_dRectOutline(byval rX1 as single, byval rY1 as single, byval rX2 as single, byval rY2 as single, byval nARGB as long, byval rSize as single)
    local nAlpha, nRed, nGreen, nBlue as byte
    zSplitColorARGB(nARGB, nAlpha, nRed, nGreen, nBlue)
    glColor4ub(nRed, nGreen, nBlue, nAlpha)
    glLineWidth(rSize)
    glBegin(%GL_LINE_LOOP)
      glVertex2f(rX1, rY1)
      glVertex2f(rX2, rY1)
      glVertex2f(rX2, rY2)
      glVertex2f(rX1, rY2)
    glEnd()
end sub

'// Draw a rectangular parallelogram.
'// Dessine un parallelipipede rectangle.
sub gl_dParallelogramRect(byval rLength as single, byval rWidth as single, byval rThickness as single, byval nARGB as long, byval nTexture as long)
    local nAlpha, nRed, nGreen, nBlue as byte
    zSplitColorARGB(nARGB, nAlpha, nRed, nGreen, nBlue)
    glColor4ub(nRed, nGreen, nBlue, nAlpha)
    if (nTexture) then
        glEnable(%GL_TEXTURE_2D)
        glBindTexture(%GL_TEXTURE_2D, nTexture)
    end if

    '// BackFace
    glBegin(%GL_QUADS)
      glTexCoord2f(1.0, 0.0): glVertex3f(-rLength,-rWidth,-rThickness) '// Bottom Right Of The Texture And Quad
      glTexCoord2f(1.0, 1.0): glVertex3f(-rLength, rWidth,-rThickness) '// Top Right Of The Texture And Quad
      glTexCoord2f(0.0, 1.0): glVertex3f( rLength, rWidth,-rThickness) '// Top Left Of The Texture And Quad
      glTexCoord2f(0.0, 0.0): glVertex3f( rLength,-rWidth,-rThickness) '// Bottom Left Of The Texture And Quad
    glEnd()
    '// TopFace
    glBegin(%GL_QUADS)
      glTexCoord2f(0.0, 1.0): glVertex3f(-rLength, rWidth,-rThickness) '// Top Left Of The Texture And Quad
      glTexCoord2f(0.0, 0.0): glVertex3f(-rLength, rWidth, rThickness) '// Bottom Left Of The Texture And Quad
      glTexCoord2f(1.0, 0.0): glVertex3f( rLength, rWidth, rThickness) '// Bottom Right Of The Texture And Quad
      glTexCoord2f(1.0, 1.0): glVertex3f( rLength, rWidth,-rThickness) '// Top Right Of The Texture And Quad
    glEnd()
    '// BottomFace
    glBegin(%GL_QUADS)
      glTexCoord2f(1.0, 1.0): glVertex3f(-rLength,-rWidth,-rThickness) '// Top Right Of The Texture And Quad
      glTexCoord2f(0.0, 1.0): glVertex3f( rLength,-rWidth,-rThickness) '// Top Left Of The Texture And Quad
      glTexCoord2f(0.0, 0.0): glVertex3f( rLength,-rWidth, rThickness) '// Bottom Left Of The Texture And Quad
      glTexCoord2f(1.0, 0.0): glVertex3f(-rLength,-rWidth, rThickness) '// Bottom Right Of The Texture And Quad
    glEnd()
    '// RightFace
    glBegin(%GL_QUADS)
      glTexCoord2f(1.0, 0.0): glVertex3f( rLength,-rWidth,-rThickness) '// Bottom Right Of The Texture And Quad
      glTexCoord2f(1.0, 1.0): glVertex3f( rLength, rWidth,-rThickness) '// Top Right Of The Texture And Quad
      glTexCoord2f(0.0, 1.0): glVertex3f( rLength, rWidth, rThickness) '// Top Left Of The Texture And Quad
      glTexCoord2f(0.0, 0.0): glVertex3f( rLength,-rWidth, rThickness) '// Bottom Left Of The Texture And Quad
    glEnd()
    '// LeftFace
    glBegin(%GL_QUADS)
      glTexCoord2f(0.0, 0.0): glVertex3f(-rLength,-rWidth,-rThickness) '// Bottom Left Of The Texture And Quad
      glTexCoord2f(1.0, 0.0): glVertex3f(-rLength,-rWidth, rThickness) '// Bottom Right Of The Texture And Quad
      glTexCoord2f(1.0, 1.0): glVertex3f(-rLength, rWidth, rThickness) '// Top Right Of The Texture And Quad
      glTexCoord2f(0.0, 1.0): glVertex3f(-rLength, rWidth,-rThickness) '// Top Left Of The Texture And Quad
    glEnd()
    '// FrontFace
    glBegin(%GL_QUADS)
      glTexCoord2f(0.0, 0.0): glVertex3f(-rLength,-rWidth, rThickness) '// Bottom Left Of The Texture And Quad
      glTexCoord2f(1.0, 0.0): glVertex3f( rLength,-rWidth, rThickness) '// Bottom Right Of The Texture And Quad
      glTexCoord2f(1.0, 1.0): glVertex3f( rLength, rWidth, rThickness) '// Top Right Of The Texture And Quad
      glTexCoord2f(0.0, 1.0): glVertex3f(-rLength, rWidth, rThickness) '// Top Left Of The Texture And Quad
    glEnd()

    if (nTexture) then glDisable(%GL_TEXTURE_2D)
end sub

'// Apply a texture onto the surface of an OpenGL rectangle, which size matches the point coordinates, using a specific ARGB color.
'// Applique une texture sur la surface d'un rectangle OpenGL dont la dimension est dtermine par les coordonnes des points opposs, en utilisant une couleur ARGB.
sub gl_dRectTexture(byval rX1 as single, byval rY1 as single, byval rX2 as single, byval rY2 as single, byval nARGB as long, byval nTexture as long)
    local nAlpha, nRed, nGreen, nBlue as byte
    zSplitColorARGB(nARGB, nAlpha, nRed, nGreen, nBlue)
    glColor4ub(nRed, nGreen, nBlue, nAlpha)
    glEnable(%GL_TEXTURE_2D)
    glBindTexture(%GL_TEXTURE_2D, nTexture)
    glBegin(%GL_QUADS)
      glTexCoord2f(0.0, 0.0): glVertex2f(rX1, rY1)
      glTexCoord2f(1.0, 0.0): glVertex2f(rX2, rY1)
      glTexCoord2f(1.0, 1.0): glVertex2f(rX2, rY2)
      glTexCoord2f(0.0, 1.0): glVertex2f(rX1, rY2)
    glEnd()
    glDisable(%GL_TEXTURE_2D)
end sub

'// To draw a cylinder.
'// Pour dessiner un cylindre.
sub gl_dCylinder(byval rX as single, byval rY as single, byval rZ as single, byval rRadius as single, byval rHeight as single, byval r1 as single, byval r2 as single, byval nARGB as long, byval nCylenderTexture as long, byval nDiskTexture as long)
    local nAlpha, nRed, nGreen, nBlue as byte
    local nQuadObj as dword
    zSplitColorARGB(nARGB, nAlpha, nRed, nGreen, nBlue)
    glColor4ub(nRed, nGreen, nBlue, nAlpha)

    glEnable(%GL_TEXTURE_2D)
    if (nCylenderTexture) then glBindTexture(%GL_TEXTURE_2D, nCylenderTexture)

    nQuadObj = gluNewQuadric()
    gluQuadricNormals(nQuadObj, %GLU_SMOOTH)
    gluQuadricTexture(nQuadObj, 1)
    glTranslatef(rX, rY, rZ)
    '//gluQuadricOrientation(nQuadObj, %GLU_OUTSIDE)
    '// r1 and r2 values are being used to draw a cone instead of a tube.
    '// r1 et r2 permettent de dessiner un cone au lieu d'un tube.
    if (r1 = 0) then r1 = 1
    if (r2 = 0) then r2 = 1
    gluCylinder(nQuadObj, rRadius * r1, rRadius * r2, rHeight, 32, 1)

    '// Draw disks.
    '// Dessine les disques.
    if (nDiskTexture) then glBindTexture(%GL_TEXTURE_2D, nDiskTexture)
    gluDisk(nQuadObj, 0, rRadius * r1, 32, 32)
    glTranslatef(0, 0, rHeight)
    gluDisk(nQuadObj, 0, rRadius * r2, 32, 32)

    gluDeleteQuadric(nQuadObj)
    glDisable(%GL_TEXTURE_2D)
end sub

'// To draw a pipe.
'// Pour dessiner un tube.
sub gl_dPipe(byval rX as single, byval rY as single, byval rZ as single, byval rRadius as single, byval rHeight as single, byval r1 as single, byval r2 as single, byval nARGB as long, byval nTexture as long)
    local nAlpha, nRed, nGreen, nBlue as byte
    local nQuadObj as dword
    zSplitColorARGB(nARGB, nAlpha, nRed, nGreen, nBlue)
    glColor4ub(nRed, nGreen, nBlue, nAlpha)

    if (nTexture) then
	      glEnable(%GL_TEXTURE_2D)
	      glBindTexture(%GL_TEXTURE_2D, nTexture)
    end if

    nQuadObj = gluNewQuadric()
    gluQuadricNormals(nQuadObj, %GLU_SMOOTH)
    gluQuadricTexture(nQuadObj, 1)
    glTranslatef(rX, rY, rZ)
    '// r1 and r2 values are being used to draw a cone instead of a tube.
    '// r1 et r2 permettent de dessiner un cone au lieu d'un tube.
    if (r1 = 0) then r1 = 1
    if (r2 = 0) then r2 = 1
    gluCylinder(nQuadObj, rRadius * r1, rRadius * r2, rHeight, 32, 1)
    gluDeleteQuadric(nQuadObj)

    if (nTexture) then glDisable(%GL_TEXTURE_2D)
end sub

'// Enable specular mapping.
'// Active le mappage speculair.
sub Enable_Specular()
    glTexGeni(%GL_S, %GL_TEXTURE_GEN_MODE, %GL_SPHERE_MAP) '// %GL_EYE_LINEAR)
    glTexGeni(%GL_T, %GL_TEXTURE_GEN_MODE, %GL_SPHERE_MAP) '// %GL_EYE_LINEAR)
    glEnable(%GL_TEXTURE_GEN_S)
    glEnable(%GL_TEXTURE_GEN_T)
end sub

'// Disable specular mapping.
'// Dsactive le mappage speculair.
sub Disable_Specular()
    glDisable(%GL_TEXTURE_GEN_S)
    glDisable(%GL_TEXTURE_GEN_T)
end sub

sub gl_DrawScene()

    static nFlip as long
    nFlip = NOT nFLIP
    if nFlip = 0 or gP.hGL = 0 then exit sub

    glClear(%GL_COLOR_BUFFER_BIT or %GL_DEPTH_BUFFER_BIT)

    local rX1, rY1, rX2, rY2, rW, rH, rRadius, rPenSize, rZoom as single
    local rChartXoff, rChartYoff, rChartZoff, rChartZoom, rChartXrot, rChartYrot as single
    local K, XX, YY, nW, nH, nARGBcolor, nAnimate, nBlack as long

    nAnimate = SendMessage(GetDlgItem(gP.hMain, %IDC_ANIMATE), %BM_GETCHECK, 0, 0)

    nW = 520
    nH = 240
    rX1 = 0: rY1 = 0: rX2 = rX1 + nW - 1: rY2 = rY1 + nH - 1

    rW = nW * 0.5
    rH = nH * 0.5

    '// Start drawing a new scene.
    '// On dbute le dessin d'une nouvelle scene.
    glLoadIdentity()

    '// Get Mousing coordinates, zooming, rotation.
    GL_ChartGetMousing(rChartXoff, rChartYoff, rChartZoff, rChartZoom, rChartXrot, rChartYrot)
    glTranslatef(rChartXoff, rChartYoff, rChartZoff)

    '// Zooming factor.
    rZoom = rChartZoom * -0.01
    glScalef(rZoom, rZoom, rZoom)

    '// Mousing rotation
    glRotatef(-rChartYrot, 1, 0, 0)
    glRotatef(-rChartXrot, 0, 1, 0)

    '// 07/05/2012
    '// Draw text
    '// Affiche un texte
    GL_DrawText(gP.hGL, gP.glfont, -200, -114, 90, "Font OpenGL 0123456789", gP.ColorGreen)'

    '// Draw a pipe
    '// Dessine un tube
    glPushMatrix()
      glRotatef(-90, 1, 0, 0) '// ROTATION
      rRadius = 10
      gl_dPipe(rX1 + 20 - rW, -40, -rH, rRadius, nH + 20, 0, 0, gP.ColorWhite, gP.texChrome)
    glPopMatrix()

    '// Draw a cylinder
    '// Dessine un cylindre
    glPushMatrix()
      rRadius = 10
      gl_dCylinder(rX1 + 20 - rW, -40, -rH, rRadius, nH + 20, 0, 0, gP.ColorWhite, gP.texChrome, gP.texRadial)
    glPopMatrix()

    '// Draw a textured rectangle.
    '// Dessine un rectangle textur.
    gl_dRectTexture(rX1 - rW, rY1 - rH, rX2 -rW, rY2 - rH, gP.ColorWhite, gP.texBuilding)

    '// Draw a rectangular outline.
    '// Dessine contour rectangle.
    rPenSize = 1: gl_dRectOutline(rX1 - rW, rY1 - rH, rX2 - rW, rY2 - rH, gP.ColorBlack, rPenSize)

    '// Draw a line.
    '// Dessine une ligne
    rPenSize = 2: gl_dLine(rX1 - rW, rY2 - rH, rX2 - rW, rY1 - rH, gP.ColorYellow, rPenSize)

    '// Draw a grid.
    '// Dessine a quadrillage.
    glPushMatrix()
      rPenSize = 2
      glTranslatef(0, 0, rPenSize)  '// TRANSLATION
      nARGBcolor = ZD_ARGB(255, 80,80,80)
      '// Draw horizontal
      for YY = 20 to nH - 20 STEP 20
          gl_dLine(rX1 - rW, nH - YY - rH - 1, rX2 - rW, nH - YY - rH - 1, nARGBcolor, rPenSize)
      next
      '// Draw vertical
      for XX = 0 to nW STEP 20
          gl_dLine(0 + XX - rW, 0 - rH, 0 + XX - rW, nH - rH, nARGBcolor, rPenSize)
      next
    glPopMatrix()

    '// Draw 5 wooden floor stage.
    '// Dessine 5 tages de plancher en bois.
    glPushMatrix()
      glRotatef(90, 1, 0, 0)      '// ROTATION
      glTranslatef(0, 12, rH)     '// TRANSLATION
      for K = 1 TO 5
          glTranslatef(0, 0, -40) '// TRANSLATION
          gl_dParallelogramRect(rW, 10, 0.5, gP.ColorWhite, gP.texFloor)
      next
    glPopMatrix()

    '// Draw asphalt ground.
    '// Dessine le sol en asphalt.
    glPushMatrix()
      glRotatef(90, 1, 0, 0)      '// ROTATION
      glTranslatef(0, 40, rH + 4) '// TRANSLATION
      gl_dParallelogramRect(rW, 40, 4, gP.ColorDarkGray, gP.texConcrete)
    glPopMatrix()

    '// Draw a wooden cube.
    '// Dessine un cube en bois.
    glPushMatrix()
      glRotatef(90, 1, 0, 0)                  '// ROTATION
      glTranslatef(rW - 20, 40 + 10, rH - 20) '// TRANSLATION
      gl_dParallelogramRect(20, 20, 20, gP.ColorWhite, gP.texFloor)
    glPopMatrix()

    '// Draw a grid.
    '// Dessine un quadrillage.
    glPushMatrix()
      glTranslatef(0, 0, rPenSize + 20)
      if (nAnimate) then nARGBcolor = gP.ColorDarkGray else nARGBcolor = gP.ColorLightGray
      '// Draw horizontal
      for YY = 20 to nH - 20 STEP 20
          gl_dLine(rX1 - rW, nH - YY - rH - 1, rX2 - rW, nH - YY - rH - 1, nARGBcolor, rPenSize)
      next
      '// Draw vertical
      for XX = 0 to nW STEP 20
          gl_dLine(0 + XX - rW, 0 - rH, 0 + XX - rW, nH - rH, nARGBcolor, rPenSize)
      next
    glPopMatrix()

    '// Draw a circle.
    '// Dessine un cercle.
    glPushMatrix()
      glTranslatef(0, 0, -20)   '// TRANSLATION
      if (nAnimate) then
          rRadius = rnd(10, rH)
          glTranslatef(rH - rRadius, rH - rRadius, 0)
      else
          rRadius = rH
      end if
      rPenSize = 2: gl_dCircle(rX1 - rW, rY1 - rH, rRadius, gP.ColorRed, rPenSize)
    glPopMatrix()

    '// Draw a cone.
    '// Dessine un cone.
    local rConeTop, rConeBase as single
    glPushMatrix()
      glTranslatef(0, 0, 40)    '// TRANSLATION
      rConeTop = 0.0001
      rConeBase = 1
      rRadius = 40: gl_dCylinder(0, 0, 0, rRadius, rRadius * 2, rConeTop, rConeBase, gP.ColorWhite, gP.texChrome, gP.texRadial)
    glPopMatrix()

    '// Draw a dome.
    '// Dessine un dome.
    glPushMatrix()
      glRotatef(90, 1, 0, 0)     '// ROTATION
      glTranslatef(80, 160, -70) '// TRANSLATION
      rRadius = 40: gl_dDome(0, 0, 0, rRadius, gP.ColorWhite, gP.texRedWood)
      '// Use a 50% transparent black color to paint the inside.
      '// On peint l'intrieur avec un noir transparent  50%.
      rRadius = 39.5: gl_dDome(0, 0, 0, rRadius, ZD_ColorARGB(128, nBlack), 0)
    glPopMatrix()

    '// Draw a sphere.
    '// Dessine une sphere.
    glPushMatrix()
      glRotatef(90, 1, 0, 0)    '// ROTATION
      glTranslatef(80, 160, 0)  '// TRANSLATION
      rRadius = 40: gl_dSphere(0, 0, 0, rRadius, gP.ColorWhite, gP.texRedWood)
    glPopMatrix()

    '// Draw a cup.
    '// Dessine une coupe.
    glPushMatrix()
      glRotatef(90, 1, 0, 0)    '// ROTATION
      glTranslatef(80, 160, 70) '// TRANSLATION
      rRadius = 40: gl_dCup(0, 0, 0, rRadius, gP.ColorWhite, gP.texRedWood)
      '// Use a 50% transparent black color to paint the inside.
      '// On peint l'intrieur avec un noir transparent  50%.
      rRadius = 39.5: gl_dCup(0, 0, 0, rRadius, ZD_ColorARGB(128, nBlack), 0)
    glPopMatrix()

    '// Draw a torus.
    '// Dessine un tore.
    glPushMatrix()
      '//glRotatef(90, 1, 0, 0) '// ROTATION
      Enable_Specular()         '// REFLEXION
      '//Enable_Wire()
      gl_dTorus(20, 150, 48, 48, gP.ColorPastelYellow, gP.texSpecular)
      '//Disable_Wire()
      Disable_Specular()
    glPopMatrix()

    '// Flush OpenGL buffer.
    glFlush()

    '// Draw the scene into the GDImage control.
    '// On dessine la scene dans le contrle GDImage.
    ZI_UpdateGLWindow(gP.hGL)
end sub

sub gl_ResizeControl()
    local rc as RECT
    GetClientRect(gP.hGL, rc)

    glViewport((rc.right - %SceneWidth) * 0.5, (rc.height - %SceneHeight) * 0.5, %SceneWidth, %SceneHeight)
    glMatrixMode(%GL_PROJECTION)
    glLoadIdentity()

    '// We force OpenGL to use a resolution that is very close to 2D pixel size
    '// as long as the constant SceneWidth and SceneHeight are equal to 1330
    gluPerspective(100, 1, 1, 1000)
    glTranslatef(0.375, 0.375, -557.65) '// 0.375 = Displacement trick FOR exact pixelization
    glMatrixMode(%GL_MODELVIEW)

    gl_DrawScene() '// OpenGL display
end sub

sub gl_InitGraph()
    local rChartXoff, rChartYoff, rChartZoff, rChartZoom, rChartXrot, rChartYrot as single
    rChartXoff = 0
    rChartYoff = 0
    rChartZoff = 0
    rChartZoom = -100
    SendMessage(GetDlgItem(gP.hMain, %IDC_ZOOMING), %TBM_SETPOS, 1, rChartZoom)
    rChartXrot = 0
    rChartYrot = 0

    GL_ChartSetMousing(rChartXoff, rChartYoff, rChartZoff, rChartZoom, rChartXrot, rChartYrot)

    SendMessage(GetDlgItem(gP.hMain, %IDC_ANIMATE), %BM_SETCHECK, %BST_UNCHECKED, 0)
end sub

function gl_WndProc(byval hWnd as dword, byval uMsg as dword, byval wParam as dword, byval lParam as long) as long

    local lp as POINT
    local nDX, nDY, nRet, nZoom as long
    local rChartXoff, rChartYoff, rChartZoff, rChartZoom, rChartXrot, rChartYrot as single
    local lpr as RECT

    GL_ChartGetMousing(rChartXoff, rChartYoff, rChartZoff, rChartZoom, rChartXrot, rChartYrot)

    select case long (uMsg)
    case %WM_LBUTTONDOWN:
         if (gP.LeftMousing = %FALSE) then
             lp.x = LO(INTEGER, lParam): lp.y = HI(INTEGER, lParam)
             gP.MouseLastPos.x = lp.x
             gP.MouseDown.x = lp.x
             gP.MouseLastPos.y = lp.y
             gP.MouseDown.y = lp.y
             gP.LeftMousing = %TRUE
         end if
         if (GetFocus() <> hWnd) then SetFocus(hWnd)

    case %WM_LBUTTONUP:
          gP.LeftMousing = %FALSE

    case %WM_RBUTTONDOWN:
         if (gP.RightMousing = %FALSE) then
             lp.x = LO(INTEGER, lParam): lp.y = HI(INTEGER, lParam)

             gP.rXcopy = rChartXoff
             gP.rYcopy = rChartYoff

             gP.nXdown = lp.x
             gP.nYdown = lp.y

             gP.RightMousing = %TRUE
         end if

    case %WM_RBUTTONUP:
         gP.RightMousing = %FALSE

    case %WM_MOUSEWHEEL:
         nZoom = SendMessage(GetDlgItem(gP.hMain, %IDC_ZOOMING), %TBM_GetPos, 0, 0)
         if (HI(INTEGER, wParam) < 0) then
             if (nZoom > -200 + 10) then nZoom -= 10
         else
             if (nZoom < -11) then nZoom += 10
         end if
         SendMessage(GetDlgItem(gP.hMain, %IDC_ZOOMING), %TBM_SetPos, 1, nZoom)
         rChartZoom = nZoom
         if (GetFocus() <> hWnd) then SetFocus(hWnd)

    case %WM_MOUSEMOVE:
         lp.x = LO(INTEGER, lParam): lp.y = HI(INTEGER, lParam)
         if (gP.LeftMousing) then
             if (ZI_IsLButtonDown() = 0) then
                 gP.LeftMousing = %FALSE
             else
                 if ((gP.nWasXL <> lp.x) or (gP.nWasYL <> lp.y)) then
                     gP.MouseDown.x = lp.x
                     gP.MouseDown.y = lp.y

                     rChartXrot = rChartXrot - (gP.MouseDown.x - gP.MouseLastPos.x)
                     rChartYrot = rChartYrot - (gP.MouseDown.y - gP.MouseLastPos.y)

                     gP.MouseLastPos.x = gP.MouseDown.x
                     gP.MouseLastPos.y = gP.MouseDown.y
                 end if
                 gP.nWasXL = lp.x: gP.nWasYL = lp.y
             end if
         elseif (gP.RightMousing) then
             if (ZI_IsRButtonDown() = 0) then
                 gP.RightMousing = %FALSE
             else
                 if ((gP.nWasXR <> lp.x) or (gP.nWasYR <> lp.y)) then
                     GetClientRect(hWnd, lpr)
                     nDX = lp.x - gP.nXdown
                     rChartXoff = gP.rXcopy - (-0.5 * nDX)
                     nDY = lp.y - gP.nYdown
                     rChartYoff = gP.rYcopy + (-0.5 * nDY)
                 end if
                 gP.nWasXR = lp.x: gP.nWasYR = lp.y
             end if
         end if
    end select

    GL_ChartSetMousing(rChartXoff, rChartYoff, rChartZoff, rChartZoom, rChartXrot, rChartYrot)

    if (gP.GLproc) then nRet = CallWindowProc(gP.GLproc, hWnd, uMsg, wParam, lParam)
    function = nRet
end function

sub gl_Initialize()

    glClearColor(0.5, 0.5, 0.5, 1) '// Grayed background, same as RGB(128,128,128)
    glClearDepth(1)
    glDepthFunc(%GL_LESS)
    glEnable(%GL_DEPTH_TEST)

    glBlendFunc(%GL_SRC_ALPHA, %GL_ONE_MINUS_SRC_ALPHA)
    glEnable(%GL_BLEND)
    glEnable(%GL_LINE_SMOOTH)

    '// Create the textures.
    dim g_mt(6) as ZGLTEXTUREX

    g_mt(0).FullName = EXEresource() + "building.jpg": g_mt(0).ID = 1: g_mt(0).Square = 0
    g_mt(1).FullName = EXEresource() + "chrome.jpg":   g_mt(1).ID = 2: g_mt(1).Square = 0
    g_mt(2).FullName = EXEresource() + "radial.jpg":   g_mt(2).ID = 3: g_mt(2).Square = 0
    g_mt(3).FullName = EXEresource() + "redwood.jpg":  g_mt(3).ID = 4: g_mt(3).Square = 0
    g_mt(4).FullName = EXEresource() + "floor.jpg":    g_mt(4).ID = 5: g_mt(4).Square = 0
    g_mt(5).FullName = EXEresource() + "concrete.jpg": g_mt(5).ID = 6: g_mt(5).Square = 0
    g_mt(6).FullName = EXEresource() + "specular.jpg": g_mt(6).ID = 7: g_mt(6).Square = 0
    ZI_MakeMultipleTexture(byval varptr(g_mt(LBOUND(g_mt))), UBOUND(g_mt) - LBOUND(g_mt) + 1)
    gP.texBuilding       = g_mt(0).texture
    gP.texChrome         = g_mt(1).texture
    gP.texRadial         = g_mt(2).texture
    gP.texRedwood        = g_mt(3).texture
    gP.texFloor          = g_mt(4).texture
    gP.texConcrete       = g_mt(5).texture
    gP.texSpecular       = g_mt(6).texture

    gP.ColorWhite        = ZD_ARGB(255, 255,255,255)
    gP.ColorGreen        = ZD_ARGB(255, 0,255,0)
    gP.ColorYellow       = ZD_ARGB(255, 255,255,0)
    gP.ColorBlack        = ZD_ARGB(255, 0,0,0)
    gP.ColorRed          = ZD_ARGB(255, 255,0,0)
    gP.ColorDarkGray     = ZD_ARGB(255, 92, 92, 92)
    gP.ColorLightGray    = ZD_ARGB(255, 192, 192, 192)
    gP.ColorPastelYellow = ZD_ARGB(155, 255,255,153)

    gl_InitGraph()

    gP.GLproc = SetWindowLong(gP.hGL, %GWL_WNDPROC, CODEPTR(gl_WndProc))

end sub

function WndProc(byval hWnd as dword, byval uMsg as dword, byval wParam as dword, byval lParam as long) as long

    local rZoomXoff, rZoomYoff, rZoomZoff, rZoomValue, rZoomXrot, rZoomYrot as single
    static hFocus as dword

    select case long (uMsg)

    case %WM_GETMINMAXINFO:
         local pMM as MINMAXINFO PTR
         pMM = lParam
         @pMM.ptMinTrackSize.x = gP.MinTrackSizeW '// minimum width size of the window.
         @pMM.ptMinTrackSize.y = gP.MinTrackSizeH '// minimum height size of the window.

    case %WM_SIZE:
         gl_ResizeControl()

    case %WM_TIMER:
         gl_DrawScene()
         function = 0: exit function

    case %WM_NCACTIVATE:
         '// Save the handle of the control that has the focus
         if (wParam = 0) then hFocus = GetFocus()

    case %WM_SETFOCUS:
         '// Post a message to set the focus later, since some
         '// Windows actions can steal it if we set it here
         if (hFocus) then
             PostMessage(hWnd, %WM_APPLYFOCUS, hFocus, 0)
             hFocus = 0
         end if

    case %WM_APPLYFOCUS:
         '// Set the focus
         if (wParam) then SetFocus(wParam)
         exit function

    case %WM_VSCROLL:
         if (GetDlgCtrlId(lParam) = %IDC_ZOOMING) then
             GL_ChartGetMousing(rZoomXoff, rZoomYoff, rZoomZoff, rZoomValue, rZoomXrot, rZoomYrot)
             rZoomValue = SendMessage(lParam, %TBM_GetPos, 0, 0)
             GL_ChartSetMousing(rZoomXoff, rZoomYoff, rZoomZoff, rZoomValue, rZoomXrot, rZoomYrot)
         end if

    case %WM_COMMAND:

         '// Messages from controls and menu items are handled here.
         select case long lo(word, wParam)
         case %IDCANCEL:
              '// if the Escape key has been pressed...
              if (hi(word, wParam) = %BN_CLICKED) then
                  '// Shut down the application by sending a WM_CLOSE message
                  SendMessage(hWnd, %WM_CLOSE, 0, 0)
                  exit function
              end if

         case %IDC_ANIMATE:
              SetFocus(gP.hGL)

         case %IDC_RESTORE:
              gl_InitGraph()
              gl_DrawScene()
              SetFocus(gP.hGL)

         end select

    case %WM_DESTROY:
         KillTimer(hWnd, %IDC_TIMER)
         PostQuitMessage(0)
         exit function

    end select

    function = DefWindowProc(hWnd, uMsg, wParam, lParam)
end function

sub InitMainTimer(byval hMain as dword)
    SetTimer(hMain, %IDC_TIMER, 0, 0)
end sub

function WinMain (byval hInstance as dword, byval hPrevInstance as dword, byval lpszCmdLine as asciiz ptr, byval nCmdShow as long) as long
    local hWnd as dword, hCtrl as dword
    local wcx as WNDCLASSEX, rc as RECT
    local zClass as asciiz * 80, szCaption as asciiz * %MAX_PATH
    local dwStyle, dwExStyle as dword
    local x, y, IsInitialized as long
    local msg as tagMSG

    gP.usefont = GetStockObject(%DEFAULT_GUI_FONT) '// Tahoma
    gP.hinstance = hInstance

    zClass     = "SCAFFOLDING"
    wcx.cbSize = sizeof(wcx)
    IsInitialized = GetClassInfoEx(hInstance, zClass, wcx)
    if (IsInitialized     = 0) then
        wcx.style         = %CS_HREDRAW or %CS_VREDRAW
        wcx.lpfnWndProc   = codeptr(WndProc)
        wcx.cbClsExtra    = 0
        wcx.cbWndExtra    = 0
        wcx.hInstance     = hInstance
        wcx.hIcon         = LoadIcon  (%NULL, byval %IDI_APPLICATION)
        wcx.hCursor       = LoadCursor(%NULL, byval %IDC_ARROW)
        wcx.hbrBackground = %COLOR_BTNSHADOW
        wcx.lpszMenuName  = %NULL
        wcx.lpszClassName = VARPTR(zClass)
        wcx.hIconSm       = LoadIcon  (%NULL, byval %IDI_APPLICATION)
        if (RegisterClassEx(wcx)) then IsInitialized = %TRUE
    end if

    if (IsInitialized) then
        dwStyle = %WS_OVERLAPPEDWINDOW or %WS_CLIPSIBLINGS or %WS_CLIPCHILDREN
        dwExStyle = %WS_EX_APPWINDOW or %WS_EX_WINDOWEDGE
        SetRect(rc, 0, 0, %ClientW, %ClientH)
        AdjustWindowRectEx(rc, dwStyle, %FALSE, dwExStyle)

        gP.MinTrackSizeW = dpi(rc.nRight - rc.nLeft )
        gP.MinTrackSizeH = dpi(rc.nBottom - rc.nTop)
        x = max&((GetSystemMetrics(%SM_CXSCREEN) - gP.MinTrackSizeW) \ 2, 0)
        y = max&((GetSystemMetrics(%SM_CYSCREEN) - gP.MinTrackSizeH) \ 2, 0)

        gP.hMain = CreateWindowEx(dwExStyle, _             '// Extended Style for The Window
                               zClass, _               '// Class Name
                               "Scaffolding", _        '// Window Title
                               dwStyle, _              '// Defined Window Style
                               x, y, _                 '// Window Position
                               gP.MinTrackSizeW, _     '// Calculate Window Width
                               gP.MinTrackSizeH, _     '// Calculate Window Height
                               %NULL, _                '// No Parent Window
                               %NULL, _                '// No Menu
                               hInstance, _            '// Instance
                               byval %NULL)            '// Dont Pass Anything To WM_CREATE

        if (gP.hMain) then

            '// Create the GDImage OpenGL window
            dwExStyle = 0
            dwStyle = %WS_CHILD or %WS_VISIBLE
            gP.hGL = WGL_CreateWindow(dwExStyle, dwStyle, 0, 0, dpi(700), dpi(600), gP.hMain, %IDC_OGLCTRL)
            if (IsWindow(gP.hGL)) then
                ZI_SetAnchorMode(gP.hGL, %ANCHOR_HEIGHT_WIDTH)
                '// Create the default GL font
                ZI_BuildGLfont(ZI_GetGLDC(gP.hGL), gP.glfont)
	              '// Setup default parameters, textures, and zooming value.
	              gl_Initialize()

                '//-------------------------------------------------------
                '// Add child controls
                hCtrl = addLabel(gP.hMain, %IDC_LABEL, "Animate red circle", 726, 25, 104, 24, %WS_CHILD or %WS_VISIBLE)
                ZI_SetAnchorMode(hCtrl, %ANCHOR_RIGHT)
                hCtrl = addButton(gP.hMain, %IDC_ANIMATE, "", 710, 21, 15, 24, %WS_CHILD or %WS_VISIBLE or %BS_AUTOCHECKBOX)
                ZI_SetAnchorMode(hCtrl, %ANCHOR_RIGHT)


                dwStyle = %WS_CHILD or %WS_VISIBLE or %TBS_VERT or %TBS_BOTTOM
                hCtrl = CreateWindowEx(0, "MSCTLS_TRACKBAR32", "", dwStyle, dpi(760), dpi(76), dpi(30), dpi(217), gP.hMain, %IDC_ZOOMING, hInstance, BYVAL %NULL)
                SendMessage(hCtrl, %TBM_SETRANGE, 1, MAK(LONG, -200, -1))
                SendMessage(hCtrl, %TBM_SETPOS, 1, -100)
                ZI_SetAnchorMode(hCtrl, %ANCHOR_RIGHT)

                hCtrl = addButton(gP.hMain, %IDC_RESTORE, "Restore", 710, 317, 120, 24, %WS_CHILD or %WS_VISIBLE)
                ZI_SetAnchorMode(hCtrl, %ANCHOR_RIGHT)

                szCaption = "|Left mouse button:|View angle.||Right mouse button:|Translation.||Mouse wheel: Zoom."
                replace "|"  with $crlf in szCaption
                hCtrl = addLabel(gP.hMain, %IDC_USINGIT, szCaption, 710, 367, 120, 120, %WS_CHILD or %WS_VISIBLE or %WS_BORDER or %SS_CENTER)
                ZI_SetAnchorMode(hCtrl, %ANCHOR_BOTTOM_RIGHT)
                '//-------------------------------------------------------

                if (skInitEngine("Scaff.sks", "")) then
                    skSkinWindow(gP.hMain, "Dock|Undock|Minimize|Maximize|Restore|Close")
                end if

                ShowWindow(gP.hMain, nCmdShow)
                UpdateWindow(gP.hMain)

                InitMainTimer(gP.hMain)

                while GetMessage(msg, %NULL, 0, 0)
                    TranslateMessage(msg)
                    DispatchMessage(msg)
                wend

                function = msg.wParam

                '// Cleanup texture from memory
                gl_DestroyTexture()
                '// Delete the default GL font.
                ZI_DeleteGLFont(gP.glfont)

            end if
        end if
    end if
end function


